@@ -80,7 +80,7 @@ meetCondition' (Y.Eq (Y.CmpNum left) (Y.CmpNum right)) subst = case (numToInt le
 meetCondition' (Y.Eq (Y.CmpAttr left) (Y.CmpAttr right)) subst = [subst | compareAttrs left right subst]
 meetCondition' (Y.Eq _ _) _ = []
 meetCondition' (Y.NF (ExMeta meta)) (Subst mp) = case M.lookup meta mp of
-  Just (MvExpression expr) ->
+  Just (MvExpression expr _) ->
     let isNf = not (matchesAnyNormalizationRule expr normalizationRules)
      in case expr of
           ExThis -> [Subst mp]
@@ -113,7 +113,7 @@ meetCondition' (Y.NF (ExMeta meta)) (Subst mp) = case M.lookup meta mp of
             _ -> False
 meetCondition' (Y.NF _) _ = []
 meetCondition' (Y.XI (ExMeta meta)) (Subst mp) = case M.lookup meta mp of
-  Just (MvExpression expr) -> meetCondition' (Y.XI expr) (Subst mp)
+  Just (MvExpression expr _) -> meetCondition' (Y.XI expr) (Subst mp)
   _ -> []
 meetCondition' (Y.XI (ExFormation _)) subst = [subst]
 meetCondition' (Y.XI ExThis) subst = []
