@@ -17,7 +17,7 @@ data MetaValue
   | MvBytes String -- !b
   | MvBindings [Binding] -- !B
   | MvFunction String -- !F
-  | MvExpression Expression -- !e
+  | MvExpression Expression Expression -- !e, the second expression is scope, which is closest formation
   | MvTail [Tail] -- !t
   deriving (Eq, Show)
 
@@ -63,44 +63,44 @@ matchAttribute ptn tgt
   | ptn == tgt = [substEmpty]
   | otherwise = []
 
-matchBinding :: Binding -> Binding -> [Subst]
-matchBinding (BiVoid pattr) (BiVoid tattr) = matchAttribute pattr tattr
-matchBinding (BiDelta pbts) (BiDelta tbts)
+matchBinding :: Binding -> Binding -> Expression -> [Subst]
+matchBinding (BiVoid pattr) (BiVoid tattr) _ = matchAttribute pattr tattr
+matchBinding (BiDelta pbts) (BiDelta tbts) _
   | pbts == tbts = [substEmpty]
   | otherwise = []
-matchBinding (BiMetaDelta meta) (BiDelta tBts) = [substSingle meta (MvBytes tBts)]
-matchBinding (BiLambda pFunc) (BiLambda tFunc)
+matchBinding (BiMetaDelta meta) (BiDelta tBts) _ = [substSingle meta (MvBytes tBts)]
+matchBinding (BiLambda pFunc) (BiLambda tFunc) _
   | pFunc == tFunc = [substEmpty]
   | otherwise = []
-matchBinding (BiMetaLambda meta) (BiLambda tFunc) = [substSingle meta (MvFunction tFunc)]
-matchBinding (BiTau pattr pexp) (BiTau tattr texp) = combineMany (matchAttribute pattr tattr) (matchExpression pexp texp)
-matchBinding _ _ = []
+matchBinding (BiMetaLambda meta) (BiLambda tFunc) _ = [substSingle meta (MvFunction tFunc)]
+matchBinding (BiTau pattr pexp) (BiTau tattr texp) scope = combineMany (matchAttribute pattr tattr) (matchExpression pexp texp scope)
+matchBinding _ _ _ = []
 
 -- Match bindings with ordering
-matchBindings :: [Binding] -> [Binding] -> [Subst]
-matchBindings [] [] = [substEmpty]
-matchBindings [] _ = []
-matchBindings ((BiMeta name) : pbs) tbs =
+matchBindings :: [Binding] -> [Binding] -> Expression -> [Subst]
+matchBindings [] [] _ = [substEmpty]
+matchBindings [] _ _ = []
+matchBindings ((BiMeta name) : pbs) tbs scope =
   let splits = [splitAt idx tbs | idx <- [0 .. length tbs]]
    in catMaybes
         [ combine (substSingle name (MvBindings before)) subst
           | (before, after) <- splits,
-            subst <- matchBindings pbs after
+            subst <- matchBindings pbs after scope
         ]
-matchBindings (pb : pbs) (tb : tbs) = combineMany (matchBinding pb tb) (matchBindings pbs tbs)
-matchBindings _ _ = []
+matchBindings (pb : pbs) (tb : tbs) scope = combineMany (matchBinding pb tb scope) (matchBindings pbs tbs scope)
+matchBindings _ _ _ = []
 
 -- Recursively go through given target expression and try to find
 -- the head expression which matches to given pattern.
 -- If there's one - build the list of all the tail operations after head expression.
 -- The tail operations may be only dispatches or applications
-tailExpressions :: Expression -> Expression -> ([Subst], [Tail])
-tailExpressions ptn tgt =
+tailExpressions :: Expression -> Expression -> Expression -> ([Subst], [Tail])
+tailExpressions ptn tgt scope =
   let (substs, tails) = tailExpressionsReversed ptn tgt
    in (substs, reverse tails)
   where
     tailExpressionsReversed :: Expression -> Expression -> ([Subst], [Tail])
-    tailExpressionsReversed ptn' tgt' = case matchExpression ptn' tgt' of
+    tailExpressionsReversed ptn' tgt' = case matchExpression ptn' tgt' scope of
       [] -> case tgt' of
         ExDispatch expr attr ->
           let (substs, tails) = tailExpressionsReversed ptn' expr
@@ -111,34 +111,34 @@ tailExpressions ptn tgt =
         _ -> ([], [])
       substs -> (substs, [])
 
-matchExpression :: Expression -> Expression -> [Subst]
-matchExpression (ExMeta meta) tgt = [substSingle meta (MvExpression tgt)]
-matchExpression ExThis ExThis = [substEmpty]
-matchExpression ExGlobal ExGlobal = [substEmpty]
-matchExpression ExTermination ExTermination = [substEmpty]
-matchExpression (ExFormation pbs) (ExFormation tbs) = matchBindings pbs tbs
-matchExpression (ExDispatch pexp pattr) (ExDispatch texp tattr) = combineMany (matchAttribute pattr tattr) (matchExpression pexp texp)
-matchExpression (ExApplication pexp pbd) (ExApplication texp tbd) = combineMany (matchExpression pexp texp) (matchBinding pbd tbd)
-matchExpression (ExMetaTail exp meta) tgt = case tailExpressions exp tgt of
+matchExpression :: Expression -> Expression -> Expression -> [Subst]
+matchExpression (ExMeta meta) tgt scope = [substSingle meta (MvExpression tgt scope)]
+matchExpression ExThis ExThis _ = [substEmpty]
+matchExpression ExGlobal ExGlobal _ = [substEmpty]
+matchExpression ExTermination ExTermination _ = [substEmpty]
+matchExpression (ExFormation pbs) (ExFormation tbs) _ = matchBindings pbs tbs (ExFormation tbs)
+matchExpression (ExDispatch pexp pattr) (ExDispatch texp tattr) scope = combineMany (matchAttribute pattr tattr) (matchExpression pexp texp scope)
+matchExpression (ExApplication pexp pbd) (ExApplication texp tbd) scope = combineMany (matchExpression pexp texp scope) (matchBinding pbd tbd scope)
+matchExpression (ExMetaTail exp meta) tgt scope = case tailExpressions exp tgt scope of
   ([], _) -> []
   (substs, tails) -> combineMany substs [substSingle meta (MvTail tails)]
-matchExpression _ _ = []
+matchExpression _ _ _ = []
 
 -- Deep match pattern to expression inside binding
-matchBindingExpression :: Binding -> Expression -> [Subst]
-matchBindingExpression (BiTau _ texp) ptn = matchExpressionDeep ptn texp
-matchBindingExpression _ _ = []
+matchBindingExpression :: Binding -> Expression -> Expression -> [Subst]
+matchBindingExpression (BiTau _ texp) ptn scope = matchExpressionDeep ptn texp scope
+matchBindingExpression _ _ _ = []
 
 -- Match expression with deep nested expression(s) matching
-matchExpressionDeep :: Expression -> Expression -> [Subst]
-matchExpressionDeep ptn tgt =
-  let matched = matchExpression ptn tgt
+matchExpressionDeep :: Expression -> Expression -> Expression -> [Subst]
+matchExpressionDeep ptn tgt scope =
+  let matched = matchExpression ptn tgt scope
       deep = case tgt of
-        ExFormation bds -> concatMap (`matchBindingExpression` ptn) bds
-        ExDispatch exp _ -> matchExpressionDeep ptn exp
-        ExApplication exp tau -> matchExpressionDeep ptn exp ++ matchBindingExpression tau ptn
+        ExFormation bds -> concatMap (\bd -> matchBindingExpression bd ptn (ExFormation bds)) bds
+        ExDispatch exp _ -> matchExpressionDeep ptn exp scope
+        ExApplication exp tau -> matchExpressionDeep ptn exp scope ++ matchBindingExpression tau ptn scope
         _ -> []
    in matched ++ deep
 
 matchProgram :: Expression -> Program -> [Subst]
-matchProgram ptn (Program exp) = matchExpressionDeep ptn exp
+matchProgram ptn (Program exp) = matchExpressionDeep ptn exp (ExFormation [BiVoid AtRho])
